def strip(str):
    return str.strip(" \t\r\n")

def eval_value(str):
    str = strip(str)

    number = True
    for c in str:
        if c not in "0123456789":
            number = False
            break

    if number:
        return int(str)

    true = ["true"]
    false = ["false"]

    if str.lower() in true:
        return True
    elif str.lower() in false:
        return False

    return str

def get_dict(str):
    dict = {}

    active_key = ""
    active_value = ""

    lines = str.splitlines()
    lines.append("1:1")

    for line in lines:
        partitions = line.split(":")
        if ":" in line and len(line.split(":")) >= 2 and len(partitions[0].split(" ")) == 1:
            if active_key != "" and active_value != "":
                if type(active_value) is str:
                    active_value = strip(active_value)
                dict[active_key] = active_value
                active_key = ""
                active_value = ""

            active_key = strip(partitions[0])
            value = strip(":".join(partitions[1:]))
            if active_key != "":
                if value == "|":
                    continue
                active_value = eval_value(value)
        elif active_key != "":
            active_value += strip(line) + "\n"

    return dict

def get_list(str):
    str += "\n--"
    lines = str.splitlines()

    dicts = list()
    s = ""
    for line in str.splitlines():
        line = strip(line)
        if line.startswith("-"):
            dicts.append(get_dict(s))
            s = ""
        elif strip(line).startswith("#"):
            continue
        else:
            s += "\n" + line

    return dicts


import re
from urllib.parse import urlparse

import apraw

from ..const import logger
from ..models import RedditItem

URL_PATTERN = re.compile(r"((https:\/\/)?((www|old|np|mod)\.)?(reddit|redd){1}(\.com|\.it){1}([a-zA-Z0-9\/_]+))")


async def get_item(reddit: apraw.Reddit, subreddits, str):
    for u in URL_PATTERN.findall(str):
        if is_url(u[0]):
            item = await get_item_from_url(reddit, subreddits, u[0])
            if item:
                return item
            else:
                continue
    return None


async def get_item_from_url(reddit: apraw.Reddit, subreddits, url):
    if url.startswith("https://mod.reddit.com/mail/all/"):
        id = url.split("/")[-1] if url.split("/")[-1] != "" else url.split("/")[-2]

        for subreddit in subreddits:
            try:
                modmail = await subreddit._subreddit.modmail(id)
                if hasattr(modmail, "subject"):
                    return RedditItem(modmail, subreddit, "url")
            except Exception as e:
                logger.error(f"Failed to fetch modmail by ID '{id}': {e}")

        return None

    item = None
    try:
        item = await reddit.comment(url=url)
    except Exception as e:
        logger.error(f"Failed to fetch comment by URL '{url}': {e}")
        try:
            item = await reddit.submission(url=url)
        except Exception as e:
            logger.error(f"Failed to fetch submission by URL '{url}': {e}")
            return None

    try:
        item_subreddit = await item.subreddit()
    except Exception as e:
        logger.error(f"Failed to retrieve item {item} subreddit: {e}")
        return None
    else:
        subreddit = None
        for sub in subreddits:
            s = await sub.get_subreddit()
            if s.id == item_subreddit.id:
                subreddit = sub
                break

        if subreddit:
            return RedditItem(item, subreddit, "url")


def is_url(url):
    check = urlparse(url)
    return check.scheme != "" and check.netloc != ""



import discord

from ..const import BOT_DISCLAIMER
from .item import RedditItem


class MessageBuilder:

    async def get_item_message(self, item: RedditItem):
        if item.type in ["submission", "comment"]:
            author_name = discord.utils.escape_markdown(await item.get_author_name())
            subreddit = discord.utils.escape_markdown(str(item.subreddit))
            return f"New {item.type} on /r/{subreddit} by /u/{author_name}!\n\n" + \
                f"{item.url}\n\n" + \
                f"**Title:** {item.item.title}\n**Body:**\n{item.body}"
        elif item.type == "modmail":
            author_name = discord.utils.escape_markdown(await item.get_author_name())
            subreddit = discord.utils.escape_markdown(str(item.subreddit))
            subject = discord.utils.escape_markdown(item.item.conversation.subject)
            return f"New message in modmail conversation '{subject}' on /r/{subreddit} by /u/{author_name}!" + \
                f"\n\n{item.body}"
        else:
            author_name = discord.utils.escape_markdown(await item.get_author_name())
            subreddit = discord.utils.escape_markdown(str(item.subreddit))
            return f"New action taken by /u/{author_name} on /r/{subreddit}: `{item.body}`"

    async def get_item_embed(self, item: RedditItem, embed_color: discord.Color = None):
        embed = discord.Embed(
            colour=embed_color or item.subreddit.banhammer.embed_color
        )

        title = ""
        if item.type in ["submission", "comment"]:
            author_name = discord.utils.escape_markdown(await item.get_author_name())
            subreddit = discord.utils.escape_markdown(str(item.subreddit))
            if item.source == "reports":
                title = f"{item.type.title()} reported on /r/{subreddit} by /u/{author_name}!"
            else:
                title = f"New {item.type} on /r/{subreddit} by /u/{author_name}!"
        elif item.type == "modmail":
            author_name = discord.utils.escape_markdown(await item.get_author_name())
            subreddit = discord.utils.escape_markdown(str(item.subreddit))
            subject = discord.utils.escape_markdown(item.item.conversation.subject)
            title = f"New message in modmail conversation '{subject}' on /r/{subreddit} by /u/{author_name}!"
        else:
            author_name = discord.utils.escape_markdown(item.item._data['mod'])
            subreddit = discord.utils.escape_markdown(str(item.subreddit))
            title = f"New action taken by /u/{author_name} on /r/{subreddit}!"

        url = item.url
        embed.set_author(name=title, url=url if url else discord.Embed.Empty)

        if item.type == "submission":
            embed.add_field(
                name="Title",
                value=discord.utils.escape_markdown(item.item.title),
                inline=False)
            if item.item.is_self:
                embed.add_field(name="Body", value=item.body, inline=False)
            else:
                embed.add_field(
                    name="URL",
                    value=discord.utils.escape_markdown(item.item.url),
                    inline=False)
            if item.source == "reports":
                embed.add_field(
                    name="Reports",
                    value="\n".join(f"{r[1]} {r[0]}" for r in item.item.user_reports),
                    inline=False)
        elif item.type == "comment":
            embed.description = item.body
        elif item.type == "modmail":
            embed.description = item.body
        elif item.type == "mod action":
            embed.description = f"Action: `{item.body}`"

        return embed

    def get_ban_message(self, item: RedditItem, ban_duration: int):
        ban_type = "permanent" if not ban_duration else "temporary"
        disclaimer = BOT_DISCLAIMER.format(item.subreddit.get_contact_url())
        return f"Our moderator team has reviewed [this post]({item.url}) and decided to give you a {ban_type} ban. " \
               f"If you wish to appeal this ban, please respond to this message.\n\n{disclaimer}"

    def format_reply(self, item: RedditItem, reply: str):
        disclaimer = BOT_DISCLAIMER.format(item.subreddit.get_contact_url())
        return f"{reply}\n\n{disclaimer}"



import setuptools

from banhammer import __author__, __license__, __tag__, __version__

with open("README.md", "r", encoding="utf8") as f:
    long_description = f.read()

setuptools.setup(
    name="Banhammer.py",
    version="{}-{}".format(__version__, __tag__) if __tag__ else __version__,
    license=__license__,
    author=__author__,
    author_email="moravrav@gmail.com",
    description="A Discord bot integration framework for Reddit moderation.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Dan6erbond/Banhammer.py",
    packages=setuptools.find_packages(include=['banhammer', 'banhammer.*']),
    package_data={'banhammer': ['WELCOME.md', 'reactions.yaml']},
    include_package_data=True,
    install_requires=[
        'discord.py>=1.1.1',
        'apraw>=0.6.6a0'
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Operating System :: OS Independent",
        "Topic :: Communications",
        "Topic :: Internet",
        "Topic :: Utilities",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
)

# classifiers can be found here: https://pypi.org/classifiers/
